@@ -1,7 +1,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
+# Most of this work is copyright (C) 2013-2021 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -26,7 +26,6 @@ from hypothesis.errors import InvalidArgument
 from hypothesis.extra import numpy as npst
 from hypothesis.internal.conjecture import utils as cu
 from hypothesis.internal.coverage import check, check_function
-from hypothesis.internal.reflection import deprecated_posargs
 from hypothesis.internal.validation import (
     check_type,
     check_valid_interval,
@@ -176,7 +175,6 @@ def range_indexes(
 
 @st.cacheable
 @st.defines_strategy()
-@deprecated_posargs
 def indexes(
     *,
     elements: Optional[st.SearchStrategy[Ex]] = None,
@@ -217,7 +215,6 @@ def indexes(
 
 
 @st.defines_strategy()
-@deprecated_posargs
 def series(
     *,
     elements: Optional[st.SearchStrategy[Ex]] = None,
@@ -333,7 +330,6 @@ class column:
     unique = attr.ib(default=False)
 
 
-@deprecated_posargs
 def columns(
     names_or_number: Union[int, Sequence[str]],
     *,
@@ -361,7 +357,6 @@ def columns(
     ]
 
 
-@deprecated_posargs
 @st.defines_strategy()
 def data_frames(
     columns: Optional[Sequence[column]] = None,
@@ -648,6 +643,11 @@ def data_frames(
                                 try:
                                     as_list[i] = fills[i]
                                 except KeyError:
+                                    if c.fill.is_empty:
+                                        raise InvalidArgument(
+                                            f"Empty fill strategy in {c!r} cannot "
+                                            f"complete row {original_row!r}"
+                                        ) from None
                                     fills[i] = draw(c.fill)
                                     as_list[i] = fills[i]
                         for k in row:
@@ -679,7 +679,13 @@ def data_frames(
                             % (original_row, len(row), len(rewritten_columns))
                         )
                     while len(row) < len(rewritten_columns):
-                        row.append(draw(rewritten_columns[len(row)].fill))
+                        c = rewritten_columns[len(row)]
+                        if c.fill.is_empty:
+                            raise InvalidArgument(
+                                f"Empty fill strategy in {c!r} cannot "
+                                f"complete row {original_row!r}"
+                            )
+                        row.append(draw(c.fill))
                     result.iloc[row_index] = row
                     break
                 else:
